syntax = "proto2";
package machinetalk;
// see README.msgid
// msgid base: 400

import "machinetalk/protobuf/nanopb.proto";
import "machinetalk/protobuf/types.proto";

message LogMessage {

    option (nanopb_msgopt).msgid = 400; // see README.msgid

    required MsgOrigin origin  = 10;
    required int32        pid  = 20;
    required MsgLevel   level  = 30;
    required string       tag  = 40; //  [(nanopb).max_size = 20];
    required string      text  = 50; //  [(nanopb).max_size = 100];
}
